@@ -43,12 +43,12 @@ class Publisher(DependencyProvider):
         self.exchange = exchange
         self.queue = queue
 
-    def get_instance(self, connection):
+    def get_instance(self, container):
         '''
         Provides a publish method to a container.
 
         Args:
-            connection: A kombu.Connection object.
+            container: A service container.
 
         Returns:
             A publish(msg) method.
@@ -61,71 +61,64 @@ class Publisher(DependencyProvider):
         if exchange is None and queue is not None:
             exchange = queue.exchange
 
-        def do_publish(msg):
+        def do_publish(msg, **kwargs):
             # TODO: would it not be better to to use a single connection
             #       per service, i.e. share it with consumers, etc?
             #       How will this work properly with eventlet?
-            with producers[connection].acquire(block=True) as producer:
-                channel = producer.channel
-                if queue is not None:
-                    maybe_declare(queue, channel)
+            with container.connection_factory() as conn:
+                with producers[conn].acquire(block=True) as producer:
+                    channel = producer.channel
+                    if queue is not None:
+                        maybe_declare(queue, channel)
 
-                elif exchange is not None:
-                    maybe_declare(exchange, channel)
+                    elif exchange is not None:
+                        maybe_declare(exchange, channel)
 
-                # TODO: should we enable auto-retry,
-                #       should that be an option in __init__?
-                producer.publish(msg, exchange=exchange)
+                    # TODO: should we enable auto-retry,
+                    #       should that be an option in __init__?
+                    producer.publish(msg, exchange=exchange, **kwargs)
 
         return do_publish
 
 
-def consume(queue, fn=None):
+def consume(queue, requeue_on_error=False):
     '''
     Decorates a method as a message consumer.
 
     Messaages from the queue will be deserialized depending on their content
     type and passed to the the decorated method.
-    When the conumer method returns without raising any exceptions, the message
-    will automatically be acknowledged.
-    If any exceptions are raised during the consumtion, the message will be
-    requeued.
-
-    It is possible to manually acknowledge a message from within the consumer.
-    To achieve this, it needs to accept a second parameter, the raw message
-    object and call ack() or requeue() on it.
+    When the conumer method returns without raising any exceptions,
+    the message will automatically be acknowledged.
+    If any exceptions are raised during the consumtion and
+    `requeue_on_error` is True, the message will be requeued.
 
     Example::
 
         @consume(...)
-        def handle_message(self, body, message):
+        def handle_message(self, body):
 
             if not self.spam(body):
-                message.requeue()
+                raise Exception('message will be requeued')
 
             self.shrub(body)
 
     Args:
         queue: The queue to consume from.
     '''
-    if fn is None:
-        return partial(consume, queue)
+    def consume_decorator(fn):
+        consumer_configs[fn] = ConsumerConfig(queue, requeue_on_error)
+        return fn
 
-    argspecs = inspect.getargspec(fn)
-    include_raw_message = len(argspecs.args) > 2
-    consumer_configs[fn] = ConsumerConfig(queue, include_raw_message)
-
-    return fn
+    return consume_decorator
 
 
 class ConsumerConfig(object):
     '''
     Stores information about a consumer-decorated method.
     '''
-    def __init__(self, queue, include_raw_message):
+    def __init__(self, queue, requeue_on_error):
         self.queue = queue
-        self.include_raw_message = include_raw_message
-        self.prefetch_count = 1
+        self.requeue_on_error = requeue_on_error
 
 
 def get_consumers(Consumer, service, on_message):
@@ -146,54 +139,11 @@ def get_consumers(Consumer, service, on_message):
             consumer_config = consumer_configs[consumer_method.im_func]
 
             consumer = Consumer(
-                            queues=[consumer_config.queue],
-                            callbacks=[partial(on_message,
-                                                consumer_config,
-                                                consumer_method)]
-                        )
-
-            consumer.qos(prefetch_count=consumer_config.prefetch_count)
-
+                queues=[consumer_config.queue],
+                callbacks=[
+                    partial(on_message, (consumer_method, consumer_config))
+                ]
+            )
             yield consumer
         except KeyError:
             pass
-
-
-def process_message(consumer_config, consumer, body, message):
-    '''
-    Processes a consumable message.
-
-    If the consumer returns without raising any exceptions, the message
-    will be acknowledged otherwise it will be requeued.
-
-    The consumer may accept just the body or the body and the raw message.
-    It may also ack() or requeue() the raw message manually, in which case
-    automatic acknowledgement/requeueing is disabled.
-
-    Args:
-        consumer_config: The configuration as defined by the consume decorator.
-        consumer: The consume-decorated method.
-
-        body: The body of the message.
-        message: The raw message.
-    '''
-    try:
-        if consumer_config.include_raw_message:
-            consumer(body, message)
-        else:
-            consumer(body)
-
-    except Exception as e:
-        if message.acknowledged:
-            log.error('failed to consume message, '
-                'cannot requeue because message already acknowledged: %s(): %s',
-                consumer, e)
-        else:
-            log.error(
-                    'failed to consume message, requeueing message: %s(): %s',
-                    consumer, e)
-            message.requeue()
-    else:
-        if not message.acknowledged:
-            message.ack()
-
